;  Copyright (c) 2005
;       Aaron Hsu
;
;  Redistribution and use in source and binary forms, with or without
;  modification, are permitted provided that the following conditions
;  are met:
;
;  1. Redistributions of source code must retain the above copyright
;     notice, this list of conditions and the following disclaimer.
;  2. Redistributions in binary form must reproduce the above copyright
;     notice, this list of conditions and the following disclaimer in the
;     documentation and/or other materials provided with the distribution.
;  3. Neither the name of Aaron Hsu nor the names of other contributors
;     may be used to endorse or promote products derived from this software
;     without specific prior written permission.
; 
;  THIS SOFTWARE IS PROVIDED BY AARON HSU AND CONTRIBUTORS ``AS IS'' AND
;  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
;  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
;  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
;  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
;  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
;  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
;  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
;  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
;  SUCH DAMAGE.
;
;;aac2ogg : directory faadargs oggencargs -> unspecified
;;takes a directory that can be understood by pathname and reads all files
;;that match for valid aac files to convert to ogg, and processes them.
(declare (usual-integrations))
(load-option 'synchronous-subprocess)
(define-structure tags title artist album genre track date)
(define (aac2ogg d faadargs oggencargs)
  (let*
      ;;->wav-name : fname -> pathname
      ;;takes a pathname and changes the pathname-type to WAV
      ((->wav-name
	(lambda (fname)
	  (make-pathname (pathname-host fname)
			 (pathname-device fname)
			 (pathname-directory fname)
			 (pathname-name fname)
			 "wav"
			 (pathname-version fname))))
       (oggenc
	(lambda (fname opts ogg-tags)
	  (run-shell-command
	   (string-append "oggenc "
			  opts
			  " -t "
			  "\"" (tags-title ogg-tags) "\""
			  " -a "
			  "\"" (tags-artist ogg-tags) "\""
			  " -l "
			  "\"" (tags-album ogg-tags) "\""
			  " -G "
			  "\"" (tags-genre ogg-tags) "\""
			  " -N "
			  "\"" (tags-track ogg-tags) "\""
			  " -d "
			  "\"" (tags-date ogg-tags) "\""
			  " "
			  "\""
			  (->namestring fname)
			  "\"")
	   'output (open-output-string))))
       (make-ogg-tags
	(lambda (iport)
	  (let loop ((line (read-line iport))
		     (ogg-tags (make-tags "" "" "" "" "" "")))
	    (if (not (eof-object? line))
		(cond ((substring? "title: " line)
		       (loop (read-line iport)
			     (make-tags (substring line 7 (string-length line))
					(tags-artist ogg-tags)
					(tags-album ogg-tags)
					(tags-genre ogg-tags)
					(tags-track ogg-tags)
					(tags-date ogg-tags))))
		      ((substring? "artist: " line)
		       (loop (read-line iport)
			     (make-tags (tags-title ogg-tags)
					(substring line 8 (string-length line))
					(tags-album ogg-tags)
					(tags-genre ogg-tags)
					(tags-track ogg-tags)
					(tags-date ogg-tags))))
		      ((substring? "album: " line)
		       (loop (read-line iport)
			     (make-tags (tags-title ogg-tags)
					(tags-artist ogg-tags)
					(substring line 7 (string-length line))
					(tags-genre ogg-tags)
					(tags-track ogg-tags)
					(tags-date ogg-tags))))
		      ((substring? "genre: " line)
		       (loop (read-line iport)
			     (make-tags (tags-title ogg-tags)
					(tags-artist ogg-tags)
					(tags-album ogg-tags)
					(substring line 7 (string-length line))
					(tags-track ogg-tags)
					(tags-date ogg-tags))))
		      ((substring? "track: " line)
		       (loop (read-line iport)
			     (make-tags (tags-title ogg-tags)
					(tags-artist ogg-tags)
					(tags-album ogg-tags)
					(tags-genre ogg-tags)
					(substring line 7 (string-length line))
					(tags-date ogg-tags))))
		      ((substring? "date: " line)
		       (loop (read-line iport)
			     (make-tags (tags-title ogg-tags)
					(tags-artist ogg-tags)
					(tags-album ogg-tags)
					(tags-genre ogg-tags)
					(tags-track ogg-tags)
					(substring line 
						   6
						   (string-length line)))))
		      (else (loop (read-line iport)
				  ogg-tags)))
		ogg-tags))))
       (convert-file
	(lambda (fname)
	  (let ((oport (open-output-string)))
	    (run-shell-command (string-append "faad -i "
					      "\""
					      (->namestring fname)
					      "\"")
			       'output oport)
	    (run-shell-command (string-append "faad "
					      faadargs
					      "\""
					      (->namestring fname)
					      "\"")
			       'output (open-output-string))
	    (oggenc (->wav-name fname)
		    oggencargs
		    (make-ogg-tags (open-input-string 
				    (get-output-string oport))))
	    (delete-file (->wav-name fname))))))
    (let loop ((curdir (directory-read d)))
      (if (not (null? curdir))
	  (cond ((or (string=? (pathname-name (car curdir))
			       "..")
		     (string=? (pathname-name (car curdir))
			       "."))
		 (loop (cdr curdir)))
		((file-directory? (car curdir))
		 (loop (append (cdr curdir)
			       (directory-read 
				(pathname-as-directory (car curdir))))))
		 ((file-regular? (car curdir))
		  (display "Converting ")
		  (display (->namestring (car curdir)))
		  (newline)
		  (convert-file (car curdir))
		  (loop (cdr curdir)))
		 (else
		  (display "Error: ")
		  (display (->namestring (car curdir)))
		  (display " is not a valid file type.")
		  (newline)
		  (loop (cdr curdir))))
		(begin
		  (display "Completed Converting ")
		  (display (->namestring d))
		  (newline))))))
  
    
    
